#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define G_UDEV_API_IS_SUBJECT_TO_CHANGE
#include <gudev/gudev.h>
#include <sys/utsname.h>

/* search path for firmare; second and fourth element are dynamically constructed
 * in missing_firmware_init(). */
const gchar* firmware_search_path[] = {
    "/lib/firmware", NULL,
    "/lib/firmware/updates", NULL,
    NULL };

static void
on_uevent (GUdevClient *client,
           gchar *action,
           GUdevDevice *device,
           gpointer user_data)
{
    const gchar* firmware_file;
    gchar *path;
    gchar **i;
    gboolean found = FALSE;

    g_debug ("firmware on_uevent: action=%s, devpath=%s", action, g_udev_device_get_sysfs_path(device));

    if (g_strcmp0 (action, "add") != 0)
	return;

    firmware_file = g_udev_device_get_property (device, "FIRMWARE");

    if (!firmware_file) {
        g_debug ("on_uevent: discarding, no FIRMWARE property");
        return;
    }

    for (i = firmware_search_path; *i && !found; ++i) {
	path = g_build_path("/", *i, firmware_file, NULL);
	g_debug("firmware on_uevent: searching for %s", path);
	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
	    g_debug("firmware on_uevent: found");
	    found = TRUE;
	}
	g_free (path);
    }

    if (found)
	return;

    g_debug("calling jockey");
    gchar* argv[] = {"jockey-gtk", "-c", NULL };
    GError* error = NULL;
    if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
	g_warning("jockey could not be called: %s", error->message);
	g_error_free (error);
    }
}

void
missing_firmware_init()
{
    const gchar* subsytems[] = {"firmware", NULL};
    
    /* build firmware search path */
    struct utsname u;
    if (uname (&u) != 0) {
	g_warning("uname() failed, not monitoring firmware");
	return;
    }
    firmware_search_path[1] = g_strdup_printf("/lib/firmware/%s", u.release);
    firmware_search_path[3] = g_strdup_printf("/lib/firmware/updates/%s", u.release);

    GUdevClient* gudev = g_udev_client_new (subsytems);
    g_signal_connect (gudev, "uevent", G_CALLBACK (on_uevent), NULL);
}
